#!/bin/bash

NAME=vort-inlet3-2023

FULL=1 # Set to 1 for a full rebuild; 0 to skip rerunning the simulations

INLETV=0.2
STOP=80
T=200
ARGS="../mpm_mac -3d 28 -last_frame $T -frame_dt 1 -mu 0 -scale_mass 3 -T $INLETV -T $STOP -reseed -analyze_energy_vort -no_check -solver_tolerance 1e-8 -warm_start -order 3 -threads 8"

#opt=("" "-regular_seeding" "-order 3" "-regular_seeding -order 3")
#opt_name=("default" "regular" "default-cubic" "regular-cubic")
opt=("")
opt_name=("default")

LO=64
HI=$LO
SKIP=32
RES=$HI

SO="-f_timepoint 1. -adv_v_timepoint 0.5 -C_INT -H_INT -polypic6"

if [ "X$FULL" = "X1" ] ; then
    rm -rf $NAME
    mkdir -p $NAME
    for o in `seq 0 $((${#opt[@]}-1))` ; do
        for r in `seq $HI -$SKIP $LO` ; do
            dt=`perl -e "print (1.0/$r)"`
            DT="-max_dt $dt -min_dt $dt"
            PVORT=""
            last_frame=$T
            if [ "X$r" = "X$MOVIE_RES" ]; then
                last_frame=$MOVIE_T
                PVORT="-particle_vort"
            fi
            LF="-last_frame $last_frame"
            echo $ARGS $DT $LF $PVORT -affine ${opt[$o]} -resolution $r -o $NAME/apic1-${opt_name[$o]}-$r
            echo $ARGS $DT $LF $PVORT $SO -resolution $r -o $NAME/qpic1-${opt_name[$o]}-$r
            echo $ARGS $DT $LF $PVORT -no_affine ${opt[$o]} -flip 1 -resolution $r -o $NAME/flip1-${opt_name[$o]}-$r
            echo $ARGS $DT $LF $PVORT -use_bdf2 -affine ${opt[$o]} -resolution $r -o $NAME/apic2-${opt_name[$o]}-$r
            echo $ARGS $DT $LF $PVORT -use_bdf2 $SO -resolution $r -o $NAME/qpic2-${opt_name[$o]}-$r
            #echo $ARGS $DT $LF $PVORT -use_bdf2 -no_affine ${opt[$o]} -flip 1 -resolution $r -o $NAME/flip2-${opt_name[$o]}-$r
        done
    done | xargs -P 16 -n 1 -d '\n' bash -c
else
    rm $NAME/*.*
fi

for s in apic1 qpic1 flip1 apic2 qpic2 ; do
    for r in `seq $HI -$SKIP $LO` ; do
        for o in `seq 0 $((${#opt[@]}-1))` ; do
            ./dambreak_parse_data.pl < $NAME/$s-${opt_name[$o]}-$r/common/log.txt > $NAME/$s-${opt_name[$o]}-$r/norms.txt
        done
    done
done

for o in ${opt_name[@]} ; do
    sed -e "s/xxx/$o/; s/rrr/$RES/; s/LLL/Vorticity/; s/DDD/vort/; s/XMAX/$T/; s/YMAX/9.0/;" vort_dambreak3_plot_2023.tex  > $NAME/plot-inlet-vort-$o.tex
    sed -e "s/xxx/$o/; s/rrr/$RES/; s/LLL/Kinetic energy/; s/DDD/ke/; s/XMAX/$T/; s/YMAX/0.02/;" dambreak_plot_2023.tex  > $NAME/plot-inlet-ke-$o.tex
    #sed -e "s/xxx/$o/; s/rrr/$RES/; s/LLL/Total energy/; s/DDD/te/; s/XMAX/10/; s/YMAX/0.3/;" te_dambreak_plot.tex  > $NAME/plot-inlet-te-$o.tex
done

cat <<EOF > $NAME/SConstruct
import os
import re
env=Environment(ENV = os.environ)
env['PSSUFFIX']=".eps"
r=re.compile(".*\.tex$")
for f in [x for x in os.listdir(".") if r.match(x)]:
    t=env.DVI(f)
    env.PostScript(t)
    env.PDF(t)
EOF
(
    cd $NAME
    scons -j 8
)
