//#####################################################################
// Copyright 2015, Craig Schroeder.
// This file is part of PhysBAM whose distribution is governed by the license contained in the accompanying file PHYSBAM_COPYRIGHT.txt.
//#####################################################################
#ifndef __STANDARD_TESTS_BASE__
#define __STANDARD_TESTS_BASE__

#include <Core/Analytic_Tests/ANALYTIC_SCALAR.h>
#include <Core/Analytic_Tests/ANALYTIC_VECTOR.h>
#include <Core/Auto_Diff/AUTO_HESS_EXT.h>
#include <Core/Implicit_Objects/ANALYTIC_IMPLICIT_OBJECT.h>
#include <Core/Parsing/PARSE_ARGS.h>
#include <Core/Random_Numbers/RANDOM_NUMBERS.h>
#include <Core/Tensors/SYMMETRIC_TENSOR.h>
#include <Deformables/Standard_Tests/DEFORMABLES_STANDARD_TESTS.h>
#include "MPM_MAC_EXAMPLE.h"
#include <functional>

namespace PhysBAM{

template<class TV> class STANDARD_TESTS;
template<class T,int d> class ISOTROPIC_CONSTITUTIVE_MODEL;
template<class TV> class POISSON_DISK;
template<class TV> struct SOURCE_PATH;

template<class TV>
class STANDARD_TESTS_BASE:public MPM_MAC_EXAMPLE<TV>
{
    typedef typename TV::SCALAR T;
    typedef VECTOR<int,TV::m> IV;
    typedef MPM_MAC_EXAMPLE<TV> BASE;
    typedef MESH_OBJECT<TV,TV::m> T_VOLUME;

public:
    using BASE::last_frame;using BASE::grid;using BASE::particles;
    using BASE::frame_title;using BASE::write_substeps_level;
    using BASE::collision_objects;using BASE::substeps_delay_frame;using BASE::viewer_dir;
    using BASE::restart;using BASE::dt;using BASE::time;
    using BASE::frame_dt;using BASE::min_dt;using BASE::max_dt;using BASE::ghost;
    using BASE::only_write_particles;using BASE::only_log;
    using BASE::cfl;using BASE::solver_tolerance;using BASE::solver_iterations;
    using BASE::threads;using BASE::weights;using BASE::stream_type;using BASE::Set_Weights;
    using BASE::Add_Collision_Object;
    using BASE::test_system;using BASE::print_matrix;
    using BASE::data_directory;using BASE::Add_Fluid_Wall;
    using BASE::test_output_prefix;using BASE::use_test_output;
    using BASE::begin_frame;using BASE::end_frame;using BASE::periodic_test_shift;using BASE::use_periodic_test_shift;
    using BASE::random;using BASE::viscosity;using BASE::mass;
    using BASE::side_bc_type;
    using BASE::Add_Callbacks;using BASE::Print_Particle_Stats;using BASE::Print_Grid_Stats;
    using BASE::bc_velocity;using BASE::bc_pressure;
    using BASE::use_object_extrap;
    using BASE::clamp_particles;using BASE::density;using BASE::velocity;
    using BASE::particle_vort;
    using BASE::xpic;using BASE::div_free_advection;
    using BASE::cubic_advection;using BASE::linear_advection;
    using BASE::position_update;
    using BASE::rk_particle_order;using BASE::interpolated_advection;
    using BASE::second_order;using BASE::opt;

    int test_number;
    int resolution;
    bool user_resolution;
    int stored_last_frame;
    bool user_last_frame;
    int order;
    int seed;
    T particles_per_cell;
    bool regular_seeding,no_regular_seeding;
    T scale_mass;
    bool override_output_directory;
    T m,s,kg;
    T unit_p,unit_rho,unit_mu;
    int forced_collision_type;
    ARRAY<std::function<void ()> > destroy;
    ARRAY<T> extra_T;
    ARRAY<int> extra_int;
    bool dump_collision_objects;
    bool test_diff;
    bool bc_periodic;
    bool use_analytic_field;
    ANALYTIC_VECTOR<TV>* analytic_velocity=0;
    ANALYTIC_SCALAR<TV>* analytic_pressure=0;
    ANALYTIC_SCALAR<TV>* analytic_level_set=0;
    bool analyze_u_modes=false;
    bool analyze_energy_vort=false;
    int dump_modes_freq=1;
    T max_ke=0;
    POISSON_DISK<TV>& poisson_disk;
    std::string analytic_u_expr,analytic_p_expr,analytic_phi_expr;
    std::string analytic_bc_types;
    RANGE<TV> analytic_grid_range=RANGE<TV>::Unit_Box();
    T arg_mu=0;
    int refine=0;
    bool fix_dt=false;
    bool second_order_pic=false;
    bool seed_threaded=false;

    STANDARD_TESTS_BASE(const STREAM_TYPE stream_type_input,PARSE_ARGS& parse_args);
    virtual ~STANDARD_TESTS_BASE();

    template<class F>
    void Add_Velocity(F f)
    {
        analytic_velocity=Make_Analytic_Vector<TV,TV::m>(f);
    }

    template<class F>
    void Add_Pressure(F f)
    {
        analytic_pressure=Make_Analytic_Scalar<TV>(f);
    }

    void Setup_Analytic_Boundary_Conditions();
    
    virtual TV Compute_Analytic_Force(const TV& X,T time) const;
    virtual T Compute_Analytic_Pressure(const TV& X,T time) const;
    virtual void Initialize_Velocity_Hessians() const;
    virtual void Initialize_Prev_Velocity_Fields() const;

    void Seed_Particles_Poisson(IMPLICIT_OBJECT<TV>& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,T particles_per_cell,
        std::function<VECTOR<T,3>(const TV&)>* color=0);

    template<class T_OBJECT> enable_if_t<!is_base_of<IMPLICIT_OBJECT<TV>,T_OBJECT>::value>
    Seed_Particles_Poisson(const T_OBJECT& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,T particles_per_cell,
        std::function<VECTOR<T,3>(const TV&)>* color=0)
    {ANALYTIC_IMPLICIT_OBJECT<T_OBJECT> obj(object);Seed_Particles_Poisson(obj,V,dV,density,particles_per_cell,color);}

    void Seed_Particles_Uniform(IMPLICIT_OBJECT<TV>& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,const GRID<TV>& seed_grid,
        std::function<VECTOR<T,3>(const TV&)>* color=0);

    template<class T_OBJECT> enable_if_t<!is_base_of<IMPLICIT_OBJECT<TV>,T_OBJECT>::value>
    Seed_Particles_Uniform(const T_OBJECT& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,const GRID<TV>& seed_grid)
    {ANALYTIC_IMPLICIT_OBJECT<T_OBJECT> obj(object);Seed_Particles_Uniform(obj,V,dV,density,seed_grid);}

    void Seed_Particles(IMPLICIT_OBJECT<TV>& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,T particles_per_cell,std::function<VECTOR<T,3>(const TV&)>* color=0);

    template<class T_OBJECT> enable_if_t<!is_base_of<IMPLICIT_OBJECT<TV>,T_OBJECT>::value>
    Seed_Particles(const T_OBJECT& object,std::function<TV(const TV&)> V,
        std::function<MATRIX<T,TV::m>(const TV&)> dV,T density,T particles_per_cell,
        std::function<VECTOR<T,3>(const TV&)>* color=0)
    {ANALYTIC_IMPLICIT_OBJECT<T_OBJECT> obj(object);Seed_Particles(obj,V,dV,density,particles_per_cell,color);}

    template<class T_OBJECT> auto
    Seed_Particles_Analytic(T_OBJECT& object,T density,T particles_per_cell)
    {return Seed_Particles(object,
            [this](const TV& X){return analytic_velocity->f(X,0);},
            [this](const TV& X){return analytic_velocity->dX(X,0);},
            density,particles_per_cell);}

    int Add_Particle(const TV& X,std::function<TV(const TV&)> V,std::function<MATRIX<T,TV::m>(const TV&)> dV,
        const T mass,const T volume,const VECTOR<T,3>& color=VECTOR<T,3>(1,1,1));
    int Add_Particle(const TV& X,const TV& V,const MATRIX<T,TV::m>& dV,const T mass,const T volume,
        const VECTOR<T,3>& color=VECTOR<T,3>(1,1,1));

    void Set_Grid(const RANGE<TV>& domain,IV resolution_scale=IV()+1,int default_resolution=32);
    void Set_Grid(const RANGE<TV>& domain,IV resolution_scale,IV resolution_padding,
        int resolution_multiple=1,int default_resolution=32);
    void Test_dV(std::function<TV(const TV&)> V,std::function<MATRIX<T,TV::m>(const TV&)> dV) const;
    void Check_Analytic_Velocity(std::function<bool(const FACE_INDEX<TV::m>&)>
        valid_face=[](const FACE_INDEX<TV::m>&){return true;}) const;
    void Velocity_Fourier_Analysis() const;
    void Energy_Vorticity_Analysis() const;
    void Add_Source(const TV& X0,const TV& n,const TV& V0,const TV& g,
        IMPLICIT_OBJECT<TV>* io,T density,T particles_per_cell,bool owns_io,std::function<bool()> cond=[](){return true;});
    void Commandline_Analytic_test();
//#####################################################################
};
}

#endif
